# Generated by Django 3.0.7 on 2020-07-30 05:26

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0011_update_proxy_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(max_length=200, null=True, unique=True)),
                ('email', models.EmailField(max_length=255, null=True, unique=True)),
                ('first_name', models.CharField(max_length=255, null=True)),
                ('last_name', models.CharField(max_length=255, null=True)),
                ('is_staff', models.BooleanField(default=False)),
                ('is_active', models.BooleanField(default=True)),
                ('date_joined', models.DateTimeField(auto_now_add=True)),
                ('last_login', models.DateTimeField(blank=True, null=True)),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import hashlib

from rest_framework.authentication import TokenAuthentication
from rest_framework.authtoken.models import Token
from rest_framework.exceptions import AuthenticationFailed

from datetime import timedelta, datetime
from django.utils import timezone
from django.conf import settings


# check if user has token
def has_a_token(user):
    try:
        token = Token.objects.get(user=user)
        return True
    except:
        return False


# create token
def create_token(user, use_recent=False):
    token = Token.objects.filter(user=user).order_by('created')
    if use_recent and token:
        reference_date = datetime.now() - timedelta(minutes=5)
        latest_token = token.last()
        if latest_token.created > reference_date:
            return latest_token

    token.delete()
    token = Token.objects.create(user=user)
    return token


# create refresh token
def create_refresh_token(token):
    key = token.key
    refresh_token = hashlib.sha256("{}-app-name".format(key)).hexdigest()
    user = token.user
    user.refresh_token = refresh_token
    user.save()
    return refresh_token


# this return left time
def expires_in(token):
    time_elapsed = datetime.now() - token.created
    left_time = timedelta(seconds=settings.REST_FRAMEWORK_TOKEN_SECONDS_EXPIRY) - time_elapsed
    return left_time


# token checker if token expired or not
def is_token_expired(token):
    return expires_in(token) < timedelta(seconds=0)


# If token is expired then it will be removed

def token_expire_handler(token):
    is_expired = is_token_expired(token)
    if is_expired:
        token.delete()
        token = None
    return is_expired, token


# ________________________________________________
# DEFAULT_AUTHENTICATION_CLASSES
class ExpiringTokenAuthentication(TokenAuthentication):
    """
    If token is expired then it will be removed
    and new one with different key will be created
    """

    def authenticate_credentials(self, key):

        try:
            token = Token.objects.get(key=key)
        except Token.DoesNotExist:
            raise AuthenticationFailed("Invalid Token")
        if not token.user.is_active:
            raise AuthenticationFailed("User is not active")

        is_expired, token = token_expire_handler(token)
        if is_expired:
            raise AuthenticationFailed("The Token is expired")
        return (token.user, token)



from django.contrib.auth.base_user import BaseUserManager
from django.utils.translation import ugettext_lazy as _


class CustomUserManager(BaseUserManager):
    def create_user(self, email, password, **extra_fields):
        """
        Creates and saves a User with the given email, date of
        birth and password.
        """

        if not email:
            raise ValueError(_('The Email must be set'))
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save()
        return user

    def create_superuser(self, email, password, **extra_fields):
        """
        Creates and saves a superuser with the given email, date of
        birth and password.
        """
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        extra_fields.setdefault('is_active', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError(_('Superuser must have is_staff=True.'))
        if extra_fields.get('is_superuser') is not True:
            raise ValueError(_('Superuser must have is_superuser=True.'))
        return self.create_user(email, password, **extra_fields)



from django.db import models

# Create your models here.
from django.db import models
from django.contrib.auth.models import AbstractUser, AbstractBaseUser
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import PermissionsMixin

from .managers import CustomUserManager


class User(AbstractBaseUser, PermissionsMixin):
    username = models.CharField(max_length=200, null=True, unique=True)
    email = models.EmailField(null=True, unique=True, max_length=255)
    first_name = models.CharField(null=True, max_length=255)
    last_name = models.CharField(null=True, max_length=255)

    is_staff = models.BooleanField(default=False)
    is_active = models.BooleanField(default=True)
    date_joined = models.DateTimeField(auto_now_add=True)
    last_login = models.DateTimeField(null=True, blank=True)

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    objects = CustomUserManager()

    def __str__(self):
        return self.email





import base64

from hashlib import sha256
from urllib.parse import urlencode

from django.shortcuts import render
from django.contrib.auth import get_user_model, login, logout
from django.http import StreamingHttpResponse
from django.shortcuts import render, redirect
from django.conf import settings
from rest_framework.reverse import reverse
from rest_framework.generics import CreateAPIView
from rest_framework import status
from rest_framework.authtoken.models import Token
from rest_framework.response import Response
from rest_framework.exceptions import NotFound
# Create your views here.

User = get_user_model()

# Create an authorizing endpoint that :
from rest_framework import serializers


def signout(request):
    logout(request)
    return redirect(settings.FRONTEND_URL)

def hash_string(string):
    encoding = 'utf-8'
    s = sha256()
    s.update(str.encode(string))
    hashed_token = base64.urlsafe_b64encode(s.digest()).decode(encoding)
    return hashed_token


class AuthSerializer(serializers.Serializer):
    redirect = serializers.CharField()


class AuthUser(CreateAPIView):
    serializer_class = AuthSerializer
    authentication_classes = [] # Improve later to limit only to one domain
    permission_classes = []

    def create(self, request, *args, **kwargs):
        # Get the data from POST
        data = request.data
        username = data.get('username')
        password = data.get('password')
        redirect_url = data.get('redirect_url')

        # Check data
        if not username:
            raise NotFound('No Username Found')
        if not password:
            raise NotFound('No Password Found')
        if not redirect_url:
            raise NotFound('No Redirect Url Found')
        try:
            user = User.objects.get(username=username)
        except:
            raise NotFound('User Not Found')

        if not user.check_password(password):
            raise NotFound('Credentials Not Matching')

        if hasattr(user, 'auth_token'):
            token = user.auth_token
            token.delete()

        token = Token.objects.create(user=user)
        hashed_token = hash_string(token.key)

        query_params = {"r": user.id,
                        "auth_token": hashed_token,
                        "redirect_url": redirect_url,
                        }
        query_string = urlencode(query_params)
        cookie_url = reverse('set-cookie')

        clientdata = {
                  "redirect": "{}{}?{}".format(settings.PUBLIC_PATH, cookie_url, query_string)
        }

        serializer = AuthSerializer(data=clientdata)
        serializer.is_valid()

        return Response(serializer.data, status=status.HTTP_201_CREATED)

# generate url for set cookie with hashed token and redirect url as query params


# Create a view that sets the cookie on GET

def set_cookie(request):
    if request.method == 'GET':
        # Get Data
        auth_hashed = request.GET.get('auth_token')
        user_id = request.GET.get('r')
        redirect_url = request.GET.get('redirect_url')

        if any([not auth_hashed, not user_id, not redirect_url]):
            return StreamingHttpResponse('Oops something went wrong please go back and login 1')


        try:
            user = User.objects.get(id=user_id)
        except:
            return StreamingHttpResponse('Oops something went wrong please go back and login')

        if not hasattr(user, 'auth_token'):
            return StreamingHttpResponse('Oops something went wrong please go back and login')

        token = user.auth_token
        checker_hash = hash_string(token.key)

        if checker_hash != auth_hashed:
            return StreamingHttpResponse('Oops something went wrong please go back and login')

        token.delete()

        login(request, user)

        return render(request, "authorize.html", context={"redirect_url": redirect_url})

